#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## Bookkun - Manage your ebooks.                                          ##
##                                                                        ##
## Copyright (C) 2012  Fabien Loison <flo AT flogisoft DOT com>           ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################


VIEWS = []


class Controllers(object):

    """Manages controllers."""

    _history = []
    _current = 0

    def add(self, controller):
        """Add a new controller.

        Args:
            controller -- The Controller.
        """
        # Remove trailing controllers
        if self._current < len(self._history) - 1:
            self._history = self._history[:self._current - 1]
        # Check the size of the history
        if len(self._history) > 10:  # FIXME config
            self._history.pop(0)
        # Add the view and update the index (_current)
        self._history.append(controller)
        self._current = len(self._history) - 1
        #
        self.refresh()
        self._update_nav()

    def prev(self):
        """Load the previous view."""
        if self._current > 0:
            self._current -= 1
        self.refresh()
        self._update_nav()

    def next(self):
        """Load the next view."""
        if self._currnet < len(self._history) - 1:
            self._current += 1
        self.refresh()
        self._update_nav()

    def get_current(self):
        """Get the current controller.

        Return:
            The current Controller.
        """
        return self._history[self._current]

    def refresh(self):
        """Refresh the current view."""
        self.get_current().refresh()

    def _update_nav(self):
        """Update the navigation buttons states."""
        pass  # TODO


class Controller(object):
    """Base class for all controllers."""

    def load(self, **kwargs):
        """Load the controller with optional args."""
        raise NotImplementedError()

    def refresh(self):
        """Refresh the current view."""
        raise NotImplementedError()
